"""
Contains information for loading tilesets from disk and information about
each tile.  Class include:

* Tile -- Stores an id and image for each tile
* Tileset -- Models a set of tiles
* TilesetFactory -- Used to load tilesets from disk and create Tileset objects

Note: We should eventually split the tileset out into the data and
factory components.
"""

import os

from pyglet.image import load
from pyglet.image import ImageGrid

from creatures.common import constants

class Tile(object):
    """
    Models an individual tile including the tile's index
    and image.
    """
    def __init__(self, id, image):
        self.id = id
        self.image = image

class Tileset(object):
    """
    Models a set of tiles.
    """
    def __init__(self, tile_width, tile_height, tiles):
        self.tile_width = tile_width
        self.tile_height = tile_height
        self.tiles = tiles

class TilesetFactory(object):
    """
    Loads a tileset from file and creates Tileset objects.
    """
    def __init__(self, img_path, tileset_descriptions):
        """
        img_path - directory containing tileset images

        tileset_descriptions - Sequence of tuples (rel_path, rows, columns, ignored_indices)
        where rel_path is the path to the tilesheet relative to img_path, rows is the number
        of rows of images, columns is the number of columns of images, and ignored_indices
        is a sequences of indices for tiles to be ignored.  Images are indexed starting
        from the bottom left.
        """
        self.img_path = img_path

        self.tileset_descriptions = tileset_descriptions

        self.tileset = None

    def load_tileset(self):
        # The tilesets are stored as a 2D grid of sprites. Sprites are images
        # with locations on screen -- they allow us to easily move and re-use
        # images without having to load separate copies of the images in memory.

        if self.tileset:
            return self.tileset

        tiles = []

        j = 0
        for filename, rows, columns, ignored in self.tileset_descriptions:
            tileset_image = load(self.img_path + os.sep + filename)
            tile_images = ImageGrid(tileset_image, columns, rows)
            for i, img in enumerate(tile_images):
                if i not in ignored:
                    tiles.append(Tile(j, img))
                    j += 1

        # width of the tiles
        tile_width = tiles[0].image.width
        # height of the tiles
        tile_height = tiles[0].image.height

        self.tileset = Tileset(tile_width, tile_height, tiles)
        return self.tileset
