"""
GUI for manipulating maps.  It allows the user to click on tiles on the map,
add tiles, and change tiles, highlights tiles that the mouse is currently hovering
over, and calls a class to render the map.  Provides facilities for creating new
maps, loading maps, and saving maps.
"""

import copy
import os

from collections import deque

from pyglet.window import key

from creatures.common import constants
from creatures.editor import dialog_utilities
from creatures.editor import drawing_utilities
from creatures.common.map_system import Grid2D
from creatures.common.map_system import MapData
from creatures.common.map_system import MapFactory
from creatures.common.map_system import TileStruct
from creatures.editor.widgets import TextButton


BARRIER = "Barrier"
BOARD_START = "BoardStart"
TELEPORT = "Teleport"

class MapEditorInterface(object):
    """
    Interface for interacting with the map.  This handles rendering the map, adding new
    tiles, and changing tile properties (barriers, board start, and teleports.)
    """
    def __init__(self, key_handler, window_height, window_width, map_renderer_factory, \
                     map_name=None):
        """
        key_handler

        window_height

        window_width

        map_renderer_factory

        map_name -- A string containing name of a map to load.  If None, a blank map is created.
        """
        self.key_handler = key_handler
        self.window_height = window_height
        self.window_width = window_width
        self.window_center = window_width / 2, window_height / 2

        self.map_factory = MapFactory(constants.MAP_DIR)
        if map_name:
            self.map = self.map_factory.load_map(map_name)
        else:
            self.map = self.map_factory.create_blank_map()

        self.map_renderer_factory = map_renderer_factory
        self.map_renderer = map_renderer_factory.create_renderer(self.map, self.window_center)

        # None means the mouse has not been initialized
        self.mouse_x = None
        self.mouse_y = None

        self.mouse_clicked = False

        # Used in add_tile()
        self.current_tile_type = None

        self.new_button = TextButton(x = 10 + 300 + 20,
                                      y = self.window_height - 10 - 50,
                                      width = 150,
                                      height = 50,
                                      text = "New",
                                      background = drawing_utilities.BLACK,
                                      border = drawing_utilities.RED,
                                      text_color = drawing_utilities.WHITE)

        self.open_button = TextButton(x = 10 + 450 + 30,
                                      y = self.window_height - 10 - 50,
                                      width = 150,
                                      height = 50,
                                      text = "Open",
                                      background = drawing_utilities.BLACK,
                                      border = drawing_utilities.RED,
                                      text_color = drawing_utilities.WHITE)

        self.save_button = TextButton(x = 10 + 600 + 40,
                                      y = self.window_height - 10 - 50,
                                      width = 150,
                                      height = 50,
                                      text = "Save",
                                      background = drawing_utilities.BLACK,
                                      border = drawing_utilities.RED,
                                      text_color = drawing_utilities.WHITE)

        self.undo_button = TextButton(x = 10 + 750 + 50,
                                      y = self.window_height - 10 - 50,
                                      width = 150,
                                      height = 50,
                                      text = "Undo",
                                      background = drawing_utilities.BLACK,
                                      border = drawing_utilities.RED,
                                      text_color = drawing_utilities.WHITE,
                                      disabled_background = drawing_utilities.GREY)

        # Nothing to undo until user performs an action
        self.undo_button.disable()

        # Used to store copies of the previous MapData objects
        # Contains tuples of the form (map_data, offsets).
        # The offsets are from the EditorMapRender and used to
        # position the map correctly
        self.previous_maps = deque()

    def update(self, dt):
        if self.key_handler[key.UP]:
            self.map_renderer.move_up()

        if self.key_handler[key.DOWN]:
            self.map_renderer.move_down()

        if self.key_handler[key.LEFT]:
            self.map_renderer.move_left()

        if self.key_handler[key.RIGHT]:
            self.map_renderer.move_right()

        if self.mouse_clicked:
            if self.new_button.is_clicked(self.mouse_x, self.mouse_y):
                self.new_map()

            elif self.save_button.is_clicked(self.mouse_x, self.mouse_y):
                map_name = dialog_utilities.show_save_map_dialog()

                if map_name is not None:
                    self.save_map(map_name)

            elif self.open_button.is_clicked(self.mouse_x, self.mouse_y):
                map_name = dialog_utilities.show_open_map_dialog()

                if map_name is not None:
                    self.open_map(map_name)

            elif self.undo_button.is_clicked(self.mouse_x, self.mouse_y):
                if self.undo_button.enabled:
                    self.undo()

            elif self.current_tile_type is not None:
                self.add_tile(self.current_tile_type)

            self.clear_mouse_clicked()

    def draw_buttons(self):
        self.undo_button.draw()
        self.new_button.draw()
        self.save_button.draw()
        self.open_button.draw()

    def draw(self):
        # order matters!
        self.map_renderer.draw()
        self.draw_buttons()

    def set_mouse_position(self, x, y):
        """
        Set current mouse position.  Used when we process mouse clicks
        and to highlight the tile that the mouse is currently hovering over.
        """
        self.mouse_x = x
        self.mouse_y = y
        self.map_renderer.set_current_tile(self.mouse_x, self.mouse_y)

    def set_mouse_clicked(self):
        """
        Indicate that the mouse has been clicked so we can process the
        event the next time update() is called.
        """
        self.mouse_clicked = True

    def clear_mouse_clicked(self):
        """
        Clear a mouse click so that we don't process a click more than one time.
        """
        self.mouse_clicked = False

    def set_current_tile_type(self, tile_type):
        """
        Set tile type that user selected.  Used to determine the
        type of tile to add.
        """
        self.current_tile_type = tile_type

    def undo(self):
        """
        Restores previous map, undoing most recent change.  Works for all changes
        performed while editor is open.
        """
        if len(self.previous_maps) != 0:
            self.map, offsets = self.previous_maps.pop()
            self.map_renderer = self.map_renderer_factory.create_renderer(self.map, offsets, \
                                                      board_start=False)
        if len(self.previous_maps) == 0:
            self.undo_button.disable()
        else:
            self.undo_button.enable()

    def store_current_map(self):
        """
        Makes a copy of the current map and stores it to enable the undo
        functionality.
        """
        offsets = self.map_renderer.x_offset, self.map_renderer.y_offset
        map_copy = copy.deepcopy(self.map)
        self.previous_maps.append((map_copy, offsets))
        self.undo_button.enable()

    def new_map(self):
        """
        Create a blank map.
        """
        self.store_current_map()
        self.map = self.map_factory.create_blank_map()
        self.map_renderer = self.map_renderer_factory.create_renderer(self.map, \
                                                                          self.window_center)

    def save_map(self, map_name):
        """
        Save the current map to file.
        """
        self.map_factory.save_map(self.map, map_name)

    def open_map(self, map_name):
        """
        Read a map from file.
        """
        self.store_current_map()
        self.map = self.map_factory.load_map(map_name)
        self.map_renderer = self.map_renderer_factory.create_renderer(self.map, \
                                                                          self.window_center)

    def add_tile(self, new_tile):
        """
        Add a new tile or change the properties (barrier, board_start, or teleport) of
        a tile at the position of the currently selected tile.

        new_tile -- If one of the following strings: "Barrier", "BoardStart", or
        "Teleport", the currently selected tile's properties are changed.  Otherwise,
        the new_tile is an integer tileset id and the current tile is replaced or new
        tiles are added to the map.  If a new tile is added, additional tiles are added
        to ensure the map is rectangular.
        """
        # Note: this stores the map even if nothing is done (e.g., user cancels a dialog)
        # Technically this is a bug because it means the undo function won't undo anything
        self.store_current_map()
        tile_col, tile_row = self.map_renderer.coordinate_to_tile(self.mouse_x, self.mouse_y)
        offsets = None
        row_shift = 0
        col_shift = 0
        if not self.map_renderer.outside_map(self.mouse_x, self.mouse_y):
            if new_tile == BARRIER:
                self.map.grid[tile_col, tile_row].barrier = \
                    not self.map.grid[tile_col, tile_row].barrier

            elif new_tile == BOARD_START:
                self.map.board_start = (tile_col, tile_row)

            elif new_tile == TELEPORT:
                if self.map.grid[tile_col, tile_row].teleport:
                    self.map.grid[tile_col, tile_row].teleport = False
                    self.map.grid[tile_col, tile_row].teleport_destination = None
                    self.map.grid[tile_col, tile_row].teleport_land = None
                else:
                    destination = dialog_utilities.show_open_map_dialog()
                    if destination is None:
                        return
                    landing_tile = dialog_utilities.show_tile_chooser_dialog(destination)
                    if landing_tile is None:
                        return
                    self.map.grid[tile_col, tile_row].teleport = True
                    self.map.grid[tile_col, tile_row].teleport_destination = destination
                    self.map.grid[tile_col, tile_row].teleport_land = landing_tile
            else:
                old_tile = self.map.grid[tile_col, tile_row]
                self.map.grid[tile_col, tile_row] = TileStruct(new_tile, \
                               barrier=old_tile.barrier, teleport=old_tile.teleport, \
                               teleport_destination=old_tile.teleport_destination)

        elif new_tile not in (BARRIER, TELEPORT, BOARD_START):
            # find rectangular boundaries defined by old map and new tile
            lowest_col = min(tile_col, 0)
            highest_col = max(tile_col, self.map.columns - 1)
            col_shift = 0 - lowest_col
            n_cols = highest_col + col_shift + 1

            lowest_row = min(tile_row, 0)
            highest_row = max(tile_row, self.map.rows - 1)
            row_shift = 0 - lowest_row
            n_rows = highest_row + row_shift + 1

            new_grid = Grid2D(cols = n_cols, rows = n_rows)

            # copy over the old tiles, shifting them to the new positions
            for (c, r), tile in iter(self.map.grid):
                new_grid[c + col_shift, r + row_shift] = tile

            # add our new tile
            c = tile_col + col_shift
            r = tile_row + row_shift
            new_grid[c, r] = TileStruct(new_tile)

            # fill in empty tiles so that board continues to be rectangular
            for (c, r), tile in iter(new_grid):
                if tile is None:
                    new_grid[c, r] = TileStruct(new_tile)

            # shift board start to new indices
            board_start = (self.map.board_start[0] + col_shift,
                                    self.map.board_start[1] + row_shift)

            new_map = MapData(grid = new_grid, rows = n_rows, cols = n_cols, \
                                  board_start = board_start)

            self.map = new_map

        self.map_renderer = \
            self.map_renderer_factory.create_shifted_renderer(self.map, self.map_renderer,
                                                              col_shift, row_shift)
