"""
GUI for selecting the current tile type to be used when manipulating
the map.
"""

from pyglet.graphics import Batch

from widgets import ImageButton
from widgets import TextButton

from creatures.editor import drawing_utilities
from creatures.common import geometry_utilities


class TileTypeChooserInterface(object):
    """
    Provides an interface for selecting tile properties (board start, teleports, barriers)
    and all available tiles in tileset.
    """
    def __init__(self, tileset_factory, window_height, window_width):
        self.tileset = tileset_factory.load_tileset()

        self.tile_width = self.tileset.tile_width
        self.tile_height = self.tileset.tile_height

        self.window_height = window_height
        self.window_width = window_width

        # used to speed up rendering
        self.batch = Batch()

        self.mouse_x = None
        self.mouse_y = None
        self.mouse_clicked = False

        # default tile type
        # Either a string for the property or an int for the tile type index
        self.current_tile_type = "Barrier"

        self.button_width = self.tile_width
        self.button_height = self.tile_height + 5

        rows = 3
        cols = 0

        x = 10 + cols * (self.button_width + 10)
        y = self.window_height - 10 - rows * self.button_height

        self.barrier_button = TextButton(x = x, y = y + 2,
                                         width = self.button_width - 2,
                                         height = self.button_height - 2,
                                         text = "B",
                                         background = drawing_utilities.BLACK,
                                         border = drawing_utilities.RED)

        cols += 1

        x = 10 + cols * (self.button_width + 10)
        y = self.window_height - 10 - rows * self.button_height

        self.teleport_button = TextButton(x = x, y = y + 2,
                                          width = self.button_width - 2,
                                          height = self.button_height - 2,
                                          text = "T",
                                          background = drawing_utilities.BLACK,
                                          border = drawing_utilities.BLUE)


        cols += 1

        x = 10 + cols * (self.button_width + 10)
        y = self.window_height - 10 - rows * self.button_height

        self.start_button = TextButton(x = x, y = y + 2,
                                       width = self.button_width - 2,
                                       height = self.button_height - 2,
                                       text = "S",
                                       background = drawing_utilities.BLACK,
                                       border = drawing_utilities.ORANGE)

        self.image_buttons = []
        for tile in self.tileset.tiles:
            cols += 1
            if cols == 18:
                cols = 0
                rows += 1
            x = 10 + cols * (self.button_width + 10)
            y = self.window_height - 10 - rows * self.button_height
            button = ImageButton(x = x, y = y, image = tile.image, batch = self.batch)
            self.image_buttons.append(button)

    def set_mouse_position(self, x, y):
        self.mouse_x = x
        self.mouse_y = y

    def set_mouse_clicked(self):
        self.mouse_clicked = True

    def clear_mouse_clicked(self):
        self.mouse_clicked = False

    def update(self, dt):
        if self.mouse_clicked:
            if self.barrier_button.is_clicked(self.mouse_x, self.mouse_y):
                self.current_tile_type = "Barrier"
            elif self.teleport_button.is_clicked(self.mouse_x, self.mouse_y):
                self.current_tile_type = "Teleport"
            elif self.start_button.is_clicked(self.mouse_x, self.mouse_y):
                self.current_tile_type = "BoardStart"
            else:
                for i, b in enumerate(self.image_buttons):
                    if b.is_clicked(self.mouse_x, self.mouse_y):
                        self.current_tile_type = self.tileset.tiles[i].id
                        break

            self.clear_mouse_clicked()

    def draw(self):
        # order matters!
        self.barrier_button.draw()
        self.teleport_button.draw()
        self.start_button.draw()
        self.batch.draw()
