import os

from pyglet.image import Animation
from pyglet.image import load
from pyglet.image import ImageGrid
from pyglet.image.atlas import TextureBin
from pyglet.sprite import Sprite

from creatures.common import constants

"""
Contains classes for modeling characters.
"""

ORIENTATION_LEFT = 0
ORIENTATION_RIGHT = 1
ORIENTATION_UP = 2
ORIENTATION_DOWN = 3
ORIENTATION_STRAIGHT = 4

class Character(object):
    def __init__(self, start_center, straight_sprite, walk_up_sprite, \
                     walk_down_sprite, walk_right_sprite, walk_left_sprite):
        self.straight_sprite = straight_sprite
        self.walk_up_sprite = walk_up_sprite
        self.walk_down_sprite = walk_down_sprite
        self.walk_right_sprite = walk_right_sprite
        self.walk_left_sprite = walk_left_sprite

        self.orientation = ORIENTATION_STRAIGHT

        #self.x, self.y = start_position

        # assume start position is given for CENTER of sprite
        # rather than bottom left
        self.x = start_center[0] - self.straight_sprite.width / 2
        self.y = start_center[1] - self.straight_sprite.height / 2

        self.SPEED = 2

        self.upper = self.y + self.straight_sprite.height
        self.right = self.x + self.straight_sprite.width

        self.width = self.straight_sprite.width
        self.height = self.straight_sprite.height

    def move_up(self, change_orientation=True):
        if change_orientation:
            self.orientation = ORIENTATION_UP
        self.y += self.SPEED

    def move_down(self, change_orientation=True):
        if change_orientation:
            self.orientation = ORIENTATION_DOWN
        self.y -= self.SPEED

    def move_left(self, change_orientation=True):
        if change_orientation:
            self.orientation = ORIENTATION_LEFT
        self.x -= self.SPEED

    def move_right(self, change_orientation=True):
        if change_orientation:
            self.orientation = ORIENTATION_RIGHT
        self.x += self.SPEED

    def move(self, position):
        self.x = position[0]
        self.y = position[1]

    def set_orientation_up(self):
        self.orientation = ORIENTATION_UP

    def set_orientation_down(self):
        self.orientation = ORIENTATION_DOWN

    def set_orientation_left(self):
        self.orientation = ORIENTATION_LEFT

    def set_orientation_right(self):
        self.orientation = ORIENTATION_RIGHT

    def set_orientation_straight(self):
        self.orientation = ORIENTATION_STRAIGHT


    def draw(self):
        self.upper = self.y + self.straight_sprite.height
        self.right = self.x + self.straight_sprite.width
        if self.orientation == ORIENTATION_STRAIGHT:
            self.straight_sprite.x = self.x
            self.straight_sprite.y = self.y
            self.straight_sprite.draw()
        elif self.orientation == ORIENTATION_UP:
            self.walk_up_sprite.x = self.x
            self.walk_up_sprite.y = self.y
            self.walk_up_sprite.draw()
        elif self.orientation == ORIENTATION_DOWN:
            self.walk_down_sprite.x = self.x
            self.walk_down_sprite.y = self.y
            self.walk_down_sprite.draw()
        elif self.orientation == ORIENTATION_RIGHT:
            self.walk_right_sprite.x = self.x
            self.walk_right_sprite.y = self.y
            self.walk_right_sprite.draw()
        elif self.orientation == ORIENTATION_LEFT:
            self.walk_left_sprite.x = self.x
            self.walk_left_sprite.y = self.y
            self.walk_left_sprite.draw()

class CharacterFactory(object):
    def __init__(self, img_dir):
        img_path = img_dir + os.sep + constants.CHARACTER_SPRITESHEET_IMG
        spritesheet_image = load(img_path)

        # assume all spritesheets have 12 x 8 tiles
        self.tiles = ImageGrid(spritesheet_image, 8, 12)

        self.period = 0.5 # sec
        self.loop = True

    def character1(self, start_center):
        walk_up_animation = Animation.from_image_sequence([self.tiles[12 * 7 + 1],
                                   self.tiles[12 * 7 + 0],
                                   self.tiles[12 * 7 + 1],
                                   self.tiles[12 * 7 + 2]],
                                   self.period,
                                   self.loop)
        bin = TextureBin()
        walk_up_animation.add_to_texture_bin(bin)

        walk_up_sprite = Sprite(walk_up_animation)

        walk_down_animation = Animation.from_image_sequence([self.tiles[12 * 5 + 1],
                                   self.tiles[12 * 5 + 0],
                                   self.tiles[12 * 5 + 1],
                                   self.tiles[12 * 5 + 2]],
                                   self.period,
                                   self.loop)
        bin = TextureBin()
        walk_down_animation.add_to_texture_bin(bin)
        walk_down_sprite = Sprite(walk_down_animation)

        walk_right_animation = Animation.from_image_sequence([self.tiles[12 * 6 + 1],
                                   self.tiles[12 * 6 + 0],
                                   self.tiles[12 * 6 + 1],
                                   self.tiles[12 * 6 + 2]],
                                   self.period,
                                   self.loop)
        bin = TextureBin()
        walk_right_animation.add_to_texture_bin(bin)
        walk_right_sprite = Sprite(walk_right_animation)


        walk_left_animation = Animation.from_image_sequence([self.tiles[12 * 4 + 1],
                                   self.tiles[12 * 4 + 0],
                                   self.tiles[12 * 4 + 1],
                                   self.tiles[12 * 4 + 2]],
                                   self.period,
                                   self.loop)
        bin = TextureBin()
        walk_left_animation.add_to_texture_bin(bin)
        walk_left_sprite = Sprite(walk_left_animation)


        straight_sprite = Sprite(self.tiles[12 * 5 + 1])

        return Character(start_center, straight_sprite, walk_up_sprite, \
                             walk_down_sprite, walk_right_sprite, walk_left_sprite)
